@@ -1,4 +1,4 @@
-/* SCCS @(#)rpartexp.c	1.1 12/13/99  */
+/*  $Id$  */
 /*
 **  rescale an Survival time so that it is essentially exponential
 **   each interval between deaths has the same number of person-years
@@ -11,11 +11,10 @@
 ** scratch
 **      wtemp(n)
 */
-
 #include "rpart.h"
-#include "rpartproto.h"
 
-void rpartexp(Sint *n2, double *y, double *wt, double *newy, double *wtemp) {
+void rpartexp(int *n2, double *y, double *wt, double *newy, double *wtemp)
+{
     int n;
     double *stop, *event;
     int i, j;
@@ -30,11 +29,11 @@ void rpartexp(Sint *n2, double *y, double *wt, double *newy, double *wtemp) {
     event= y+n;
 
     temp =0;
-    for (i=n-1; i>=0; i--) {   
+    for (i=n-1; i>=0; i--) {
 	temp += wt[i];
 	wtemp[i] = temp;   /* sum of weights */
-	}
-	
+    }
+
     last = 0;
     ltime =0;   /* last event time */
     rtime =0;   /* rescaled time, cumulative */
@@ -45,7 +44,7 @@ void rpartexp(Sint *n2, double *y, double *wt, double *newy, double *wtemp) {
 	psum =0;
 	for (i=last; (i<n) && (event[i]==0); i++) {
 	    psum += wt[i]*(stop[i]-ltime);    /*partial intervals */
-	    }
+	}
 
 	/*
 	** Found it (or the end of the data)
@@ -53,26 +52,25 @@ void rpartexp(Sint *n2, double *y, double *wt, double *newy, double *wtemp) {
 	if (i>n) {  /* no more deaths */
 	    for (i=last; i<n; i++) newy[i] = rtime;
 	    last=n;
-	    }
+	}
 	else { /* rescale this interval */
 	    /*
 	    ** count up the sum of the weighted deaths
 	    */
 	    dsum=0;
 	    time= stop[i];
-	    for (; i<n && event[i]==1 && stop[i]==time; i++) {
+	    for (; (i<n) && (event[i]==1) && (stop[i]==time); i++) {
 		dsum += wt[i];  /*tied deaths */
-		}
-	    
+	    }
+
 	    tsum = (wtemp[i] + dsum)*(time-ltime) + psum;
 	    scale = dsum/tsum;   /* scaling factor */
-	    
-	    for (j=last; j<i; j++) 
+
+	    for (j=last; j<i; j++)
 		newy[j] = rtime + (stop[j]-ltime)*scale;
 	    rtime += (time-ltime)*scale;
 	    last =i;
 	    ltime = time;
-	    }
 	}
     }
-	    
+}
