@@ -18,29 +18,23 @@
 **  maxcat  : max # categories, in any given categorical variable
 **  error   : possible error message
 **  parms   : vector of input parameters, initializers for the splitting rule
+**  savesort: saved version of rp.sorts
 */
 #include <math.h>
-#include <stdio.h>
 #include "rpart.h"
 #include "node.h"
-#include "rpartS.h"
 #include "rpartproto.h"
 
 #define DEBUG 0
-#if DEBUG
-static int debug =0;    /*if it is odd, print out every tree */
-                        /*if >= 2, print out every risk value we see */
-#endif
+static int debug = 0;   /*if it is odd, print out every tree */
+			/*if >= 2, print out every risk value we see */
 
-/* Next line only if mainline version */
-#ifdef MAIN
-extern char *xname[];
-#endif
-
-void xval(int n_xval,  struct cptable *cptable_head,  Sint *x_grp, 
-	  int maxcat,  char **error,                  double * parms)
-    {
-    int i,j,k, jj;
+void xval(int n_xval, struct cptable *cptable_head, int *x_grp,
+	  int maxcat,  char **error, double * parms, int *savesort)
+{
+    int i,j,k, ii, jj;
+    int last;
+    int xgroup;
     double *xtemp, *xpred;
     int    *savew;
     double *cp;
@@ -49,10 +43,9 @@ void xval(int n_xval,  struct cptable *cptable_head,  Sint *x_grp,
     struct cptable *cplist;
     double temp;
     double old_wt, total_wt;
-    int *which;
 
     alphasave = rp.alpha;
-    which = rp.which;
+
     /*
     ** Allocate a set of temporary arrays
     */
@@ -60,48 +53,73 @@ void xval(int n_xval,  struct cptable *cptable_head,  Sint *x_grp,
     xpred = xtemp + rp.num_unique_cp;
     cp    = xpred + rp.num_unique_cp;
     savew = (int *)   CALLOC(rp.n, sizeof(int));
-    for (i=0; i<rp.n; i++) savew[i] = rp.which[i];
+    for (i=0; i<rp.n; i++) savew[i] = rp.which[i]; /*restore at the end */
 
     /*
     ** Make the list of CPs that I will compare against
     */
     cp[0] = 10* cptable_head->cp;    /*close enough to infinity */
-    i=1;
-    for (cplist= cptable_head; i<rp.num_unique_cp; cplist = cplist->forward) {
+    i = 1;
+    for (cplist= cptable_head; i < rp.num_unique_cp; cplist = cplist->forward) {
 	cp[i] = sqrt(cplist->cp * (cplist->forward)->cp);
 	i++;
-	}
-    total_wt =0;
-    for (i=0; i<rp.n; i++) total_wt += rp.wt[i];
+    }
+    total_wt = 0;
+    for (i = 0; i < rp.n; i++) total_wt += rp.wt[i];
     old_wt = total_wt;
 
     /*
     ** do the validations
     */
-    for (i=0; i<n_xval; i++) {
+    k = 0; /* -Wall */
+    for (xgroup=0; xgroup<n_xval; xgroup++) {
 	/*
-	** mark the "leave out" data as fictional node 0, the rest as node 1
+	** restore rp.sorts, with the data for this run at the top
+	**   this requires one pass per variable
 	*/
-	k=0;
-	temp =0;
-	for (j=0; j<rp.n; j++) {
-	    if (x_grp[j]==(i+1)) {
-		which[j] =0;
-		}
-	    else {
-		which[j] =1;
-		rp.ytemp[k] = rp.ydata[j];
-		rp.wtemp[k] = rp.wt[j];
-		k++;
-		temp += rp.wt[j];
+	for (j=0; j<rp.nvar; j++) {
+	    k=0;
+	    for (i=0; i<rp.n; i++) {
+		ii = savesort[j*rp.n +i];
+		if (ii<0) ii = -(1+ii);  /* missings move too */
+		if (x_grp[ii]!=(xgroup+1)) {
+		    /*
+		    ** this obs is left in --
+		    **  copy to the front half of rp.sorts
+		    */
+		    rp.sorts[j][k] = savesort[j*rp.n + i];
+		    k++;
 		}
 	    }
+	}
 
+	/*
+	**  Fix up the y vector, and save a list of "left out" obs
+	**   in the tail, unused end of rp.sorts[0][i];
+	*/
+	last=k;
+	k=0;
+	temp =0;
+	for (i=0; i<rp.n; i++) {
+	    rp.which[i] =1;  /*everyone starts in group 1 */
+	    if (x_grp[i] == (xgroup +1)) {
+		rp.sorts[0][last] = i;
+		last++;
+	    } else {
+		rp.ytemp[k] = rp.ydata[i];
+		rp.wtemp[k] = rp.wt[i];
+		temp += rp.wt[i];
+		k++;
+	    }
+	}
+
+	/* at this point k = #obs in the xval group */
 	/* rescale the cp */
 	for (j=0; j<rp.num_unique_cp; j++) cp[j] *= temp/old_wt;
 	rp.alpha *= temp/old_wt;
 	old_wt = temp;
 
+
 	/*
 	** partition the new tree
 	*/
@@ -111,45 +129,43 @@ void xval(int n_xval,  struct cptable *cptable_head,  Sint *x_grp,
 	(*rp_eval)(k, rp.ytemp, xtree->response_est, &(xtree->risk),
 		   rp.wtemp);
 	xtree->complexity = xtree->risk;
-	partition(1, xtree, &temp);
+	partition(1, xtree, &temp, 0, k);
 	fix_cp(xtree, xtree->complexity);
-#ifdef MAIN
-	if (debug%2 ==1) print_tree(xtree, 1, xname,0,0);
-#endif
+
 	/*
 	** run the extra data down the new tree
 	*/
-	for (j=0; j<rp.n; j++) {
-	    if (which[j]==0) {
-		rundown(xtree, j, cp, xpred, xtemp);
+	for (i=k; i<rp.n; i++) {
+	    j = rp.sorts[0][i];
+	    rundown(xtree, j, cp, xpred, xtemp);
 #if DEBUG > 1
-		if (debug >1) {
-		   jj = j+1;
-		   printf("\nObs %d, y=%f \n", jj, rp.ydata[j][0]);
-		}
+	    if (debug > 1) {
+		jj = j+1;
+		Rprintf("\nObs %d, y=%f \n", jj, rp.ydata[j][0]);
+	    }
 #endif
-		/* add it in to the risk */
-		cplist = cptable_head;
-		for (jj = 0; jj<rp.num_unique_cp; jj++) {
-		    cplist->xrisk += xtemp[jj] * rp.wt[j];
-		    cplist->xstd  += xtemp[jj]*xtemp[jj] * rp.wt[j];
+	    /* add it in to the risk */
+	    cplist = cptable_head;
+	    for (jj = 0; jj<rp.num_unique_cp; jj++) {
+		cplist->xrisk += xtemp[jj] * rp.wt[j];
+		cplist->xstd  += xtemp[jj]*xtemp[jj] * rp.wt[j];
 #if DEBUG > 1
-		    if (debug>1) printf("  cp=%f, pred=%f, xtemp=%f\n",
-					cp[jj]/old_wt, xpred[jj], xtemp[jj]);
+		if (debug > 1)
+		    Rprintf("  cp=%f, pred=%f, xtemp=%f\n",
+			    cp[jj]/old_wt, xpred[jj], xtemp[jj]);
 #endif
-		    cplist = cplist->forward;
-		    }
-		}
+		cplist = cplist->forward;
 	    }
-	free_tree(xtree, 1);
 	}
+	free_tree(xtree, 1);
+    }
 
     for (cplist = cptable_head; cplist!=0; cplist=cplist->forward) {
 	cplist->xstd = sqrt( cplist->xstd -
-				      cplist->xrisk* cplist->xrisk/total_wt);
-	}
+			     cplist->xrisk* cplist->xrisk/total_wt);
+    }
     rp.alpha=alphasave;
     for (i=0; i<rp.n; i++) rp.which[i] = savew[i];
     Free(savew);
     Free(xtemp);
-    }
+}
